if not SimpleSelfRebuff then return end
local SimpleSelfRebuff = SimpleSelfRebuff
SimpleSelfRebuff:_rev("$Revision: 71645 $")

-------------------------------------------------------------------------------
-- Externals
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("SimpleSelfRebuff")

local STATE_SET      = SimpleSelfRebuff.STATE_SET
local STATE_FOUND    = SimpleSelfRebuff.STATE_FOUND
local STATE_EXPIRING = SimpleSelfRebuff.STATE_EXPIRING
local STATE_QUIET    = SimpleSelfRebuff.STATE_QUIET
local STATE_DONTCAST = SimpleSelfRebuff.STATE_DONTCAST

local bit_band = bit.band
local bit_bor = bit.bor

local STATE_CASTABLE_MASK  = bit_bor(STATE_SET, STATE_DONTCAST)
local STATE_CASTABLE_VALUE = STATE_SET

local new = SimpleSelfRebuff.new
local del = SimpleSelfRebuff.del

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local db

local bindingButton
local TEH_BUTTON_NAME = "SimpleSelfRebuff_MouseWheelBindingButton"

local pendingBuff
local boundBuff

local boundKeys = {}
local isBound = false

local updateEvents = {
	'SimpleSelfRebuff_MonitoringEnabled',
	'SimpleSelfRebuff_StateChanged',
	'PLAYER_CONTROL_GAINED',
	'PLAYER_REGEN_ENABLED',
	'PLAYER_AURAS_CHANGED'
}

-------------------------------------------------------------------------------
-- Module declaration
-------------------------------------------------------------------------------

local MouseWheelBinding = SimpleSelfRebuff:NewModule('MouseWheelBinding')

MouseWheelBinding.optionName = L["Mouse wheel bindings"]

MouseWheelBinding.options = {
	zoom = {
		type = 'toggle',
		name = L["Zoom when casting"],
		desc = L["Enable zooming when using the mousewheel to cast."],
		get = function() return db.zoom end,
		set = function(v) db.zoom = v end
	},
}

function MouseWheelBinding:OnModuleInitialize()
	self:RegisterDefaults('profile', {
		zoom = true,
	})
	db = self.db.profile
end

function MouseWheelBinding:OnModuleEnable()
	self:RegisterBucketEvent(updateEvents, 0.05, 'UpdateBinding')
	
	self:RegisterEvent('PLAYER_REGEN_DISABLED', 'SetBinding', false)
	self:RegisterEvent('SimpleSelfRebuff_MonitoringDisabled', 'SetBinding', false)
	self:RegisterEvent('UPDATE_BINDINGS', 'KeyBindingChanged')
	self:KeyBindingChanged()
		
	self:UpdateBinding()
end

function MouseWheelBinding:OnModuleDisable()
	self:SetBinding(false)
end

function MouseWheelBinding:KeyBindingChanged()
	boundKeys = del(boundKeys)
	boundKeys = new("MOUSEWHEELUP", "MOUSEWHEELDOWN", "SHIFT-MOUSEWHEELUP", "SHIFT-MOUSEWHEELDOWN", GetBindingKey('SIMPLE_REBUFF'))
end

local function checkBuffStatus(buff)
	if not buff or not buff:IsUsable() then
		return nil
	end
	local state, timeLeft = buff:GetState()
	if bit_band(state, STATE_SET) == 0 or bit_band(state, STATE_DONTCAST) ~= 0 then
		-- Unset or ignored buff
		return nil
	elseif bit_band(state, STATE_FOUND) == 0 then
		-- Missing buff
		return buff, 1, state, nil
	elseif bit_band(state, STATE_EXPIRING) ~= 0 then
		-- Expiring buff
		return buff, 0, state, timeLeft
	else
		-- Buff is ok
		return nil
	end		
end

function MouseWheelBinding:UpdateBinding()
	self:Debug('updating binding');
	if self.core:IsMonitoringActive() then
		
		local pendingPriority, pendingState, pendingTimeLeft
		if pendingBuff then
			pendingBuff, pendingPriority, pendingState, pendingTimeLeft = checkBuffStatus(pendingBuff)
		end
	
		for category, state, expected, actual, timeLeft in self.core:IterateCategories(STATE_SET) do
			local expected, priority, state, timeLeft = checkBuffStatus(expected)
			if expected then
				if
					not pendingBuff
					or (priority > pendingPriority)
					or (priority == pendingPriority and timeLeft and pendingTimeLeft and timeLeft < pendingTimeLeft)
				then
					pendingBuff, pendingPriority, pendingState, pendingTimeLeft = expected, priority, state, timeLeft
				end
			end
		end
		
		self:Debug('pendingBuff=%q, prio=%q, state=%q, timeLeft=%q', pendingBuff, pendingPriority, pendingState, pendingTimeLeft)
		
		self:SetBinding(pendingBuff)		
	else
		self:SetBinding(false)
	end	
end

function MouseWheelBinding:SetBinding(buff)

	if buff then
		
		if InCombatLockdown() then
			return
		end
		
		if not bindingButton then
			bindingButton = CreateFrame("Button", TEH_BUTTON_NAME, UIParent, "SecureActionButtonTemplate")
			bindingButton:RegisterForClicks("LeftButtonUp")
			bindingButton:SetPoint("CENTER",0,0)
			bindingButton:SetScript("PreClick", function(...) return self:OnButtonPreClick(...) end)
			bindingButton:SetScript("PostClick", function(...) return self:OnButtonPostClick(...) end)
			bindingButton:Hide()
			
			self:ClearButtonBindings()
		end
		
		-- Binding
		if not isBound then
			self:Debug('Binding mousewheel')
			for i,key in ipairs(boundKeys) do
				SetOverrideBindingClick(bindingButton, true, key, TEH_BUTTON_NAME, key)
			end
			isBound = true
		end
		
		boundBuff = buff
		self:Debug('Bound buff: '..boundBuff.name)
		
	else
		
		if InCombatLockdown() then
			self:Debug('Plan unbinding for end of combat')
			self:ScheduleLeaveCombatAction('SetBinding', false)
		else
			if bindingButton and isBound then
				self:Debug('Unbinding mousewheel')
				self:ClearButtonBindings()
				ClearOverrideBindings(bindingButton)
			end
			isBound = false
			boundBuff = nil
		end
		
	end

end

function MouseWheelBinding:OnButtonPreClick()
	if not InCombatLockdown() then
		self:Debug('OnButtonPreClick-START')
		if boundBuff then
			if not boundBuff:IsInCooldown() and boundBuff:IsUsable() then
				self:Debug('Setting up casting for '..boundBuff.name)
				boundBuff:SetupSecureButton(bindingButton)
			else
				self:Debug(boundBuff.name.." in cooldown or not usable")
				self:ClearButtonBindings()
			end
		else
			self:Debug("No bound buff")
			self:ClearButtonBindings()
		end
		self:Debug('OnButtonPreClick-END')
	end
end

function MouseWheelBinding:OnButtonPostClick(button, key)
	if db.zoom then
		if key == 'MOUSEWHEELUP' then
			CameraZoomIn(1)
		elseif key == 'MOUSEWHEELDOWN' then
			CameraZoomOut(1)
		end
	end

	if boundBuff and not InCombatLockdown() then
		if self:ClearButtonBindings() then
			boundBuff = nil
		end	
	end
end

function MouseWheelBinding:ClearButtonBindings()
	if bindingButton and bindingButton:GetAttribute('type') then
		self:Debug('Clearing up binding button')
		bindingButton:SetAttribute('type', nil)
		bindingButton:SetAttribute('spell', nil)			
		bindingButton:SetAttribute('unit', nil)			
		bindingButton:SetAttribute('target-slot', nil)					
		return true
	end
end

function MouseWheelBinding:DummyBinding()
	self:Debug("Do nothing")
end

function MouseWheelBinding:OnTextUpdate(plugin)
	if pendingBuff then
		plugin:SetText(pendingBuff.name)
		plugin:SetIcon(self.core.spellIcons[pendingBuff.name])
	end
end

local attrs = { "type", "target-slot", "unit", "spell" }
function MouseWheelBinding:OnTooltipUpdate(Tablet)

	if not self.core:IsDebugging() then
		return
	end

	local cat = Tablet:AddCategory('columns', 2)
	
	cat:AddLine('text', 'MouseWheelBinding')
	cat:AddLine('text', 'pendingBuff:', 'text2', pendingBuff and pendingBuff.name or "nil")
	if pendingBuff then
		cat:AddLine('text', 'pendingBuff:IsUsable():', 'text2', pendingBuff:IsUsable() and "true" or "false")
	end
	cat:AddLine('text', 'boundBuff:', 'text2', boundBuff and boundBuff.name or "nil")
	if boundBuff then
		cat:AddLine('text', 'boundBuff:IsUsable():', 'text2', boundBuff:IsUsable() and "true" or "false")
	end
	cat:AddLine('text', 'isBound:', 'text2', isBound and "true" or "false")
	cat:AddLine('text', 'IsMonitoringActive:', 'text2', self.core:IsMonitoringActive() and "true" or "false")	
	cat:AddLine('text', 'bindingButton:', 'text2', bindingButton and "created" or "nil") 
	
	if bindingButton then
		for i,name in ipairs(attrs) do
			cat:AddLine('text', 'bindingButton-'..name..':', 'text2', bindingButton:GetAttribute(name) or "nil")
		end
	end

end
